function [Rf,dRf] = equations(sys,Uf,dUf)
% Equations of the system of the form Rf(Uf) = C + L(Uf) + Q(Uf,Uf).

u = Uf(1:sys.neq);         % x,y
lambda = Uf(sys.neq+1);    % lambda
Ua = Uf(sys.neq+2:end);  % v,r

R = zeros(sys.neq,1);
Ra = zeros(sys.neq_aux,1);
dRf = zeros(sys.neq_tot,1);

alpha = sys.parameters.alpha;

% Equations of the system
%  0 = y
%  0 = -alpha y + v*r - x
R(1) = u(2);
R(2) = -alpha*u(2) + Ua(1)*Ua(2) - u(1);

% Auxiliary variables
%     0 = v - lambda y
%     0 = r - 1 + x^2
Ra(1) = Ua(1) -lambda*u(2);
Ra(2) = Ua(2) -1 - u(1)^2 ;

%%% Concatenation
Rf = [R;Ra];

end