function [Fpnl_tot] = Fpnl(obj,U,p)
% Compute  the r.h.s Fpnl at order p from U (Taylor class)

H    = obj.H;
nz_tot   = obj.nz_tot;

DHp1=2*H+1 ;

Fpnl_fourier=zeros(DHp1,nz_tot);
Fpnl_phase = 0;
aux_var = zeros(2,1);
cond_ini = zeros(1,nz_tot);

%%% As Q is symmetric, the sum is separated in two parts to optimize the
%%% time-computation.
for r=1:floor((p-1)/2)
    Ur  =get(U,'coefk',r) ;
    [Zr_tot,omegar,lambdar,omega2r,lambdaomegar] = obj.get_Ztot(Ur);
    Upmr=get(U,'coefk',p-r);
    [Zpmr_tot,omegapmr,lambdapmr,omega2pmr,lambdaomegapmr] = obj.get_Ztot(Upmr);
    
    % Vectorial computation of all the complex forms.
    Zr_compl = obj.real_to_compl(Zr_tot,'full');
    Zpmr_compl = obj.real_to_compl(Zpmr_tot,'full');
    if ~isempty(obj.id) || ~isempty(obj.idd) || ~isempty(obj.id1) ; Zd_pmr = obj.D(Zpmr_tot); Zd_r = obj.D(Zr_tot); end
    if ~isempty(obj.idd) ; Zdd_pmr = obj.D(Zd_pmr); Zdd_r = obj.D(Zd_r); end
    
    Zr_t0 = sum(Zr_tot(1:H+1,:),1);
    Zpmr_t0 = sum(Zpmr_tot(1:H+1,:),1);
    
    % Loops on the lists.
    for i=1:numel(obj.iforce2)
        Fpnl_fourier(obj.hforce2(i),obj.iforce2(i))= Fpnl_fourier(obj.hforce2(i),obj.iforce2(i)) - 2*obj.vforce2(i)*lambdar*lambdapmr ;
    end
    
    for i=1:numel(obj.ic2)
        Fpnl_fourier(1,obj.ic2(i))= Fpnl_fourier(1,obj.ic2(i)) + 2*obj.vc2(i)*lambdar*lambdapmr ;
    end
    
    for i=1:numel(obj.il1)
        Fpnl_fourier(:,obj.il1(i))= Fpnl_fourier(:,obj.il1(i)) + obj.vl1(i)*(lambdar*Zpmr_tot(:,obj.jl1(i)) + lambdapmr*Zr_tot(:,obj.jl1(i)));
    end
    
    % Specific treatment, where obj.uiq = unique(obj.iq) (see SystHBQ)
    Prods = (2*obj.vq)'.*( obj.Prod(Zr_compl(:,obj.jq),Zpmr_compl(:,obj.kq),'complex') );
    
    for i=1:numel(obj.uiq)
        Fpnl_fourier(:,obj.uiq(i))= Fpnl_fourier(:,obj.uiq(i)) + sum(Prods(:,(obj.iq==obj.uiq(i))),2);
    end
    
    %%% Does not seem to show better results because of the cost of unique function.
%     iq_update = obj.iq;
%     i_iq = obj.i_iq;
%     uiq = obj.uiq;
%     while ~isempty(uiq)
%         Fpnl_fourier(:,uiq)= Fpnl_fourier(:,uiq) + Prods(:,i_iq); % vectorial affectation.
%         Prods(:,uiq)=[];
%         iq_update(i_iq)=[];
%         [uiq,i_iq,~] = unique(iq_update);
%     end
    
    for i=1:numel(obj.id)
        Fpnl_fourier(:,obj.id(i))= Fpnl_fourier(:,obj.id(i)) + obj.vd(i)*(omegar*Zd_pmr(:,obj.jd(i))+omegapmr*Zd_r(:,obj.jd(i))) ;
    end
    
    for i=1:numel(obj.idd)
        Fpnl_fourier(:,obj.idd(i))= Fpnl_fourier(:,obj.idd(i)) + obj.vdd(i)*(omega2r*Zdd_pmr(:,obj.jdd(i))+omega2pmr*Zdd_r(:,obj.jdd(i))) ;
    end
    
    for i=1:numel(obj.id1)
        Fpnl_fourier(:,obj.id1(i))= Fpnl_fourier(:,obj.id1(i)) + obj.vd1(i)*(lambdaomegar*Zd_pmr(:,obj.jd1(i))+lambdaomegapmr*Zd_r(:,obj.jd1(i))) ;
    end
    
    % definition of omega2 and lambdaomega :
    aux_var = aux_var - [2*omegar*omegapmr; (lambdar*omegapmr + lambdapmr*omegar)];
    
    if ~isempty(obj.idl)
        for i=1:numel(obj.idq)
            Fpnl_fourier(:,obj.idq(i))= Fpnl_fourier(:,obj.idq(i)) + ...
                obj.vdq(i)*( obj.Prod(Zr_compl(:,obj.jdq(i)),obj.D(Zpmr_compl(:,obj.kdq(i)),'complex'),'complex') + ...
                obj.Prod(Zpmr_compl(:,obj.jdq(i)),obj.D(Zr_compl(:,obj.kdq(i)),'complex'),'complex'));
        end
        
        cond_ini(obj.idq)= cond_ini(obj.idq) + obj.vdq'.*((p-r)/p*Zr_t0(obj.jdq).*Zpmr_t0(obj.kdq) + r/p*Zpmr_t0(obj.jdq).*Zr_t0(obj.kdq));
        
    if strcmp(obj.subtype,'conservative')
            %%% sum(ui(0)^2 + ui'(0)^2).
            %%%% To change 10/09/2018
         Zdr_t0 = sum((1:H)'.*Zr_tot(H+2:DHp1,:),1);
         Zdpmr_t0= sum((1:H)'.*Zpmr_tot(H+2:DHp1,:),1);
%         Fpnl_phase = Fpnl_phase + ...
%             2*(Zr_t0(obj.zi_phase)-lambdar)*(Zpmr_t0(obj.zi_phase)-lambdapmr) + ... 
%             2*Zdr_t0(obj.zi_phase)*Zdpmr_t0(obj.zi_phase);
        %%% sum(ui'(0)^2).
        Fpnl_phase = Fpnl_phase + 2*sum(Zdr_t0.*Zdpmr_t0);
    end
    
    end
end


if mod(p,2) == 0
    Ur=get(U,'coefk',p/2);
    [Zr_tot,omegar,lambdar,omega2r,lambdaomegar] = obj.get_Ztot(Ur);
    
    % Vectorial computation of all the complex forms.
    Zr_compl = obj.real_to_compl(Zr_tot,'full');
    if ~isempty(obj.id) || ~isempty(obj.idd) || ~isempty(obj.id1) ; Zd_r = obj.D(Zr_tot); end
    if ~isempty(obj.idd) ; Zdd_r = obj.D(Zd_r); end
    
    Zr_t0 = sum(Zr_tot(1:H+1,:),1);
    
    % Loops on the lists.
    for i=1:numel(obj.iforce2)
        Fpnl_fourier(obj.hforce2(i),obj.iforce2(i))= Fpnl_fourier(obj.hforce2(i),obj.iforce2(i)) - obj.vforce2(i)*lambdar^2 ;
    end
    
    for i=1:numel(obj.ic2)
        Fpnl_fourier(1,obj.ic2(i))= Fpnl_fourier(1,obj.ic2(i)) + obj.vc2(i)*lambdar^2 ;
    end
    
    for i=1:numel(obj.il1)
        Fpnl_fourier(:,obj.il1(i))= Fpnl_fourier(:,obj.il1(i)) + (obj.vl1(i)*lambdar)*Zr_tot(:,obj.jl1(i));
    end
    
    % Specific treatment, where obj.uiq = unique(obj.iq) (see SystHBQ)
    Prods = obj.vq'.*( obj.Prod(Zr_compl(:,obj.jq),Zr_compl(:,obj.kq),'complex') );
    
    for i=1:numel(obj.uiq)
        Fpnl_fourier(:,obj.uiq(i))= Fpnl_fourier(:,obj.uiq(i)) + sum(Prods(:,(obj.iq==obj.uiq(i))),2);
    end
    
    %%% Does not seem to show better results because of the cost of unique function.
%     iq_update = obj.iq;
%     i_iq = obj.i_iq;
%     uiq = obj.uiq;
%     while ~isempty(uiq)
%         Fpnl_fourier(:,uiq)= Fpnl_fourier(:,uiq) + Prods(:,i_iq); % vectorial affectation.
%         Prods(:,uiq)=[];
%         iq_update(i_iq)=[];
%         [uiq,i_iq,~] = unique(iq_update);
%     end
    
    for i=1:numel(obj.id)
        Fpnl_fourier(:,obj.id(i))= Fpnl_fourier(:,obj.id(i)) + (obj.vd(i)*omegar)*Zd_r(:,obj.jd(i)) ;
    end
    
    for i=1:numel(obj.idd)
        Fpnl_fourier(:,obj.idd(i))= Fpnl_fourier(:,obj.idd(i)) + (obj.vdd(i)*omega2r)*Zdd_r(:,obj.jdd(i)) ;
    end
    
    for i=1:numel(obj.id1)
        Fpnl_fourier(:,obj.id1(i))= Fpnl_fourier(:,obj.id1(i)) + (obj.vd1(i)*lambdaomegar)*Zd_r(:,obj.jd1(i)) ;
    end
    
    % definition of omega2 and lambdaomega :
    aux_var = aux_var - [omegar^2; lambdar*omegar];
    
    if ~isempty(obj.idl)
        for i=1:numel(obj.idq)
            Fpnl_fourier(:,obj.idq(i))= Fpnl_fourier(:,obj.idq(i)) + ...
                obj.vdq(i)*( obj.Prod(Zr_compl(:,obj.jdq(i)),obj.D(Zr_compl(:,obj.kdq(i)),'complex'),'complex'));
        end
        
        cond_ini(obj.idq)= cond_ini(obj.idq) + obj.vdq'.*(1/2*Zr_t0(obj.jdq).*Zr_t0(obj.kdq));
    end
    
    if strcmp(obj.subtype,'conservative')
            %%% sum(ui(0)^2 + ui'(0)^2).
            %%%% To change 10/09/2018
         Zdr_t0 = sum((1:H)'.*Zr_tot(H+2:DHp1,:),1);
%         Fpnl_phase = Fpnl_phase + ...
%             (Zr_t0(obj.zi_phase)-lambdar)*(Zr_t0(obj.zi_phase)-lambdar) + ... 
%             Zdr_t0(obj.zi_phase)*Zdr_t0(obj.zi_phase);
        %%% sum(ui'(0)^2).
        Fpnl_phase = Fpnl_phase + sum(Zdr_t0.^2);
    end
end
    
% transform the matrix Fpnl to a column vector and add 0 for the phase
% equation, omega2,.. eqautions
Fpnl_fourier(1,obj.idq) = cond_ini(obj.idq);
Fpnl_fourier = reshape(Fpnl_fourier,nz_tot*DHp1,1);

Fpnl = [Fpnl_fourier(1:obj.neq-1);Fpnl_phase];
Fpnl_aux = [Fpnl_fourier(obj.neq:end);aux_var];
Fpnl_tot = [Fpnl;Fpnl_aux];
