% function Hand = plotbranchHBM(sys,Section,Idisp,Hdisp,bifpara_str,fig)
%    plots the amplitude of the harmonics Hdisp of the variables Idisp computed
%    by the Manlab/HBM method as a function of the bifurcation parameter
%    specified by bifpara_str ( = 'omega' or 'lambda').
%
%    It also displays the bifurcation points:
%    - B for a simple bifurcation (saddle-node, pitchfork...)
%    - PD for period doubling bifurcation
%    - NS for Neimark-Sacker bifurcation
%
%
%    By L. Guillot and O. Thomas / Nov. 2018


function Hand = plotbranchHBMbif(sys,Section,Idisp,Hdisp,bifpara_str,fig)

% Activate axis
% =============

if floor(fig)==fig
    figure(fig)
else
    disp('fig must be a figure handle')
end
hold on;

Nb=length(Idisp);
Hand=[];
Handleg=[];

% Color for successive plots.
Col = dispcolorML;

switch bifpara_str
    case 'omega'
        ind_bifpara = sys.neq;
    case 'lambda'
        ind_bifpara = sys.neq+1;
end

st1 = Section.drawtype_init;
st2 = Section.drawtype_end;

bifpara = Section.Upp(ind_bifpara,:);

H = sys.H;
DHp1 = 2*H+1;

if ~strcmp(st1,st2)
    ind_change = Section.ind_change-1;
    
    bifparastab = Section.Ustab(ind_bifpara);
    
    bifpara = [bifpara(1:ind_change-1),bifparastab,bifpara(ind_change:end)];
end

for ii=1:Nb
    idisp = Idisp(ii);
    hdisp = Hdisp(ii);
    
    cdisp = Col(ii,:);
    
    if idisp <= sys.nz
        I0   = (idisp-1)*DHp1+1;
        Icos = (idisp-1)*DHp1+1+hdisp;
        Isin = (idisp-1)*DHp1+1+H+hdisp;
    else
        I0   = sys.neq+1+ (idisp-1-sys.nz)*DHp1+1;
        Icos = sys.neq+1+ (idisp-1-sys.nz)*DHp1+1+hdisp;
        Isin = sys.neq+1+ (idisp-1-sys.nz)*DHp1+1+H+hdisp;
    end
    
    Hampl   = sqrt(Section.Upp(Icos,:).^2+Section.Upp(Isin,:).^2);
    
    if strcmp(st1,st2)
        hand = plot(bifpara,Hampl,st1,'Color',cdisp);
        Hand = [Hand;hand];
    else
        Hampl_stab   = sqrt(Section.Ustab(Icos,:).^2+Section.Ustab(Isin,:).^2);
        Hampl = [Hampl(1:ind_change-1),Hampl_stab,Hampl(ind_change:end)];
        
        
        hand = plot(bifpara(1:ind_change),Hampl(1:ind_change),st1,'Color',cdisp);
        Hand = [Hand;hand];
        hand = plot(bifpara(ind_change:end),Hampl(ind_change:end),st2,'Color',cdisp);
        Hand = [Hand;hand];
        hand = plot(bifpara(ind_change),Hampl(ind_change),'.','Color',cdisp);
        Hand = [Hand;hand];
        
        text(bifparastab,Hampl_stab,Section.Eigen.type,'margin',3,'verticalalignment','Bottom')
        box on;
    end
    
    Handleg(ii) = plot(NaN,NaN,'-','Color',cdisp);
    Leg{ii}=['z_{',num2str(idisp),'}H',num2str(hdisp)];
end

legend(Handleg,Leg,'location','best'); % Slows the plotting procedure

ylabel('Harmonics amplitude')
xlabel(['\' bifpara_str])



end