% function Hand = plotfloquetexpHBM(sys,Section,bifpara_str,fig)
%    plots the Floquet exponents at the end of the Section
%    as a function of the bifurcation parameter
%    specified by bifpara_str ( = 'omega' or 'lambda').
%
%    It also displays the Floquet at the bifurcation points.
%
%    By L. Guillot and O. Thomas / Nov. 2018

function Hand = plotfloquetexpHBM(sys,Section,bifpara_str,fig)

% Activate axis
% =============
if floor(fig)==fig
    figure(fig)
else
    disp('fig must be a figure handle')
end
subplot(2,2,1)
hold on
subplot(2,2,3)
hold on
subplot(2,2,[2 4])
hold on

Hand = [];

% Color for successive plots.
Col = dispcolorML(sys.nz);

switch bifpara_str
    case 'omega'
        bifpara = Section.Uend(sys.neq);
        try bifpara_stab = Section.Ustab(sys.neq); catch; end
    case 'lambda'
        bifpara = Section.Uend(sys.neq+1);
        try bifpara_stab = Section.Ustab(sys.neq+1); catch; end
end

floq_exp = Section.Eigen_end.values;
floq_exp = sort(floq_exp,'ComparisonMethod','real');

%%% Test if there is a bifurcation in the Section
if ~strcmp(Section.Eigen.type,'nothing')
    floq_exp_stab = Section.Eigen.values;
    floq_exp_stab = sort(floq_exp_stab,'ComparisonMethod','real');
end

Nb = length(floq_exp);

for i=1:Nb
    cdisp = Col(i,:);
    
    % Real part
    subplot(2,2,1)
    hand = plot(bifpara,real(floq_exp(i)),'.','Color',cdisp);
    Hand = [Hand;hand];
    
    % Imaginary part
    subplot(2,2,3)
    hand = plot(bifpara,imag(floq_exp(i)),'.','Color',cdisp);
    Hand = [Hand;hand];
    
    % Plot the exponent in the complex plane.
    subplot(2,2,[2 4])
    hand = plot(real(floq_exp(i)),imag(floq_exp(i)),'.','Color',cdisp);
    Hand = [Hand;hand];
    
    %%% Test if there is a bifurcation in the Section
    if ~strcmp(Section.Eigen.type,'nothing')
        % Real part
        subplot(2,2,1)
        hand = plot(bifpara_stab,real(floq_exp_stab(i)),'p','Color',cdisp);
        Hand = [Hand;hand];
        
        % Imaginary part
        subplot(2,2,3)
        hand = plot(bifpara_stab,imag(floq_exp_stab(i)),'p','Color',cdisp);
        Hand = [Hand;hand];
        
        % Plot the exponent in the complex plane.
        subplot(2,2,[2 4])
        hand = plot(real(floq_exp_stab(i)),imag(floq_exp_stab(i)),'p','Color',cdisp);
        Hand = [Hand;hand];
    end
end

subplot(2,2,1)
xlabel(['\' bifpara_str])
ylabel('Real part of the Floquet exponents');
subplot(2,2,3)
xlabel(['\' bifpara_str])
ylabel('Imaginary part of the Floquet exponents');
subplot(2,2,[2 4])
xlabel('Real part of the Floquet exponents');
ylabel('Imaginary part of the Floquet exponents');


end